package Petal::Hash::Utils;

use 5.006;
use Petal::Hash;
use strict;

our $VERSION = '0.01';


##
## Define the enclosed packages inside the Petal Modifiers hash
$Petal::Hash::MODIFIERS->{'uc:'} = 'Petal::Hash::UpperCase';
$Petal::Hash::MODIFIERS->{'lc:'} = 'Petal::Hash::LowerCase';

package Petal::Hash::UpperCase;
sub process {
	my $class = shift;
	my $hash  = shift;
	my $args  = shift;

	my $result = $hash->fetch($args);
	return uc($result);
}

package Petal::Hash::LowerCase;
sub process {
	my $class = shift;
	my $hash  = shift;
	my $args  = shift;

	my $result = $hash->fetch($args);
	return lc($result);
}


##
## Alternatively, use anonymous subs to insert new modifiers into the Petal
## Modifiers hash. Note that we do not get the $class value in this format.

#
# Uppercase the first letter of the string
$Petal::Hash::MODIFIERS->{'uc_first:'} = sub {
	my ($hash, $args) = @_;
	my $result = $hash->fetch ($args);
	#die "Result - $result";
	return "\u$result";
};

#
# Convert date from yyyy-mm-dd|yyyy/mm/dd|yyyymmdd to mm/dd/yyyy
# Arguments:
# 	$date - the date to be converted
# 	[$sep] - separator to use in new string (defaults to /)
$Petal::Hash::MODIFIERS->{'us_date:'} = sub {
	my ($hash, $args) = @_;
	my ($year,$mon,$day);
	my ($date, $sep) = $hash->fetch ($args);
	$sep ||= '/';
	if ($date =~ /[-|\/]/) {
		($year,$mon,$day) = split(/[-|\/]/, $date);
	}
	else {
		($year,$mon,$day) = $date =~ /(\d{4})(\d{2})(\d{2})/;
	}
	return sprintf("%02d$sep%02d$sep%04d", $mon,$day,$year);
};

#
# Perform an OR comparison
$Petal::Hash::MODIFIERS->{'or:'} = sub {
	my ($hash, $args) = @_;
	my ($arg1, $arg2) = split /\s/, $args;
	my $h1 = $hash->fetch($arg1);
	my $h2 = $hash->fetch($arg2);
	return ($h1 || $h2) ? 1:0;
};

#
# Return a list of keys for a hashref
$Petal::Hash::MODIFIERS->{'keys:'} = sub {
	my ($hash, $args) = @_;
	my $result = $hash->fetch( $args );
	die "$args is not a hash!" unless ref($result) eq 'HASH';
	return [ keys %$result ];
};

#
# Return a list of keys/values for a hashref
$Petal::Hash::MODIFIERS->{'each:'} = sub {
	my ($hash, $args) = @_;
	my $result = $hash->fetch( $args );
	die "$args is not a hash!" unless ref($result) eq 'HASH';
	return [ map { { key => $_, val => $result->{$_} } } keys %$result ];
};
1;

__END__

=head1 NAME

Petal::Hash::Utils - Useful modifiers for the Petal template attribute language.

=head1 SYNOPSIS

	see L<Petal> documentation

=head1 DESCRIPTION

The modifiers include with this module will be made automatically available
when Petal is used. For further details, see the "Advanced Petal" section of
the L<Petal> documentation 

=head1 MODIFIERS

=head2 lc:

	<p tal:content="lc: $string">lower</p>

Make the entire string lowercase.


=head2 uc:

	<p tal:content="uc: $string">upper</p>

Make the entire string uppercase.


=head2 uc_first:

	<p tal:content="uc_first: $string">uc_first</p>

Make the first letter of the string uppercase.

=head2 us_date:

	<p tal:content="us_date: $date">2003-09-05</p>

Convert an international date stamp (e.g., yyyymmdd, yyyy-mm-dd, yyyy/mm/dd) to US format (mm/dd/yyyy).


=head2 or:

	<p tal:if="or: $first $second">first or second = <span tal:replace="or: $first $second">or</span></p>

Compare two strings for truthfulness according to Perl (e.g., non-zero,
non-empty string).


=head2 keys:

	<ul>
		<li tal:repeat="key keys: $hash_ref"><span tal:replace="key">key</span></li>
	</ul>

Return a list of keys for a hashref. Note: It appears that values cannot be
accessed with dynamic keys. If you need the keys and values, use "each:".


=head2 each:

	<ul>
		<li tal:repeat="item each: $hash_ref"><span tal:replace="item/key">key</span> => <span tal:replace="item/val">value</span>
		</li>
	</ul>

Return a list of keys/values for a hashref



=head1 AUTHOR

William McKee, E<lt>william@knowmad.comE<gt>

This module free software and is distributed under the same license as Perl
itself. Use it at your own risk.

Thanks to Jean-Michel for making Petal available to the Perl community and to
Steve Purkis for his keys: and each: modifiers.

=head1 SEE ALSO

L<perl>, L<Petal>.

=cut
